/*************************************************************************************
 *
 *	Alerts4P: popup alert generator
 *
 *	v1.0 - 07/11/2018
 *
 *	Copyright Rolf van Gelder, CAGE Web Design, Eindhoven, NL
 *
 *************************************************************************************/
package com.cage.alerts4p;

import processing.core.*; 

public class Alerts4P extends PApplet {

	// PROCESSING APPLET
	PApplet app;

	// VERSION OF THE LIBRARY
	String version     = "1.0";
	String releaseDate = "07/11/2018";
	
	// ENABLE / DISABLE ALERTS
	boolean hideAlerts = false;

	// MESSAGE TO SHOW
	String currentMsg  = "";

	// EXPIRATION TIME OF THE MESSAGE
	int endingMS       = 0;

	// DEFAULT SETTINGS
	int textSize       = 24;
	int boxHeight      = textSize + 16;
	int horSpacing     = 20;
	int txtColor       = color(0);
	int bgColor        = color(255, 220);
	int strokeColor    = color(140);
	int strokeWeight   = 1;
	int rectRadius     = 10;
	int xPos;
	int yPos;
	int duration       = 1500;


	/****************************************************************************************
	* 
	*   Default Constructor
	*
	*	@param pApp		Processing applet
	*
	****************************************************************************************/
	public Alerts4P(PApplet pApp) {
		// PROCESSING APPLET		
		app  = pApp;
		// CENTERED ON SCREEN
		xPos = app.width >> 1;
		yPos = app.height >> 1;
	} // Alerts4P()

   
	/*********************************************************************************
	 *
	 *	Specific Constructor With Position
	 *
	*	@param pApp		Processing applet	 
	 *	@param pXPos	x position for the alert
	 *	@param pYPos	y position for the alert	 
	 *
	 *********************************************************************************/ 
	public Alerts4P(PApplet pApp, int pXPos, int pYPos) {
		// PROCESSING APPLET
		app  = pApp;
		// SPECIFIC POSITION
		xPos = pXPos;
		yPos = pYPos;
	} // Alerts4P()


	/****************************************************************************************
	* 
	*   Specific Constructor With Several Parameters
	*
	*	@param pApp			Processing applet	
	*	@param pTextSize	font size
	*	@param pHorSpacing	horizontal spacing
	*	@param pTxtColor	text color
	*	@param pBgColor		background color
	*	@param pRectRadius	radius of the corners
	*
	****************************************************************************************/
	public Alerts4P(PApplet pApp, int pTextSize, int pHorSpacing, int pTxtColor, int pBgColor, int pRectRadius) {
		// PROCESSING APPLET		
		app        = pApp;
		// CENTERED ON SCREEN
		xPos       = app.width  >> 1;
		yPos       = app.height >> 1;		
		textSize   = pTextSize;
		boxHeight  = textSize + 16;
		horSpacing = pHorSpacing;
		txtColor   = pTxtColor;
		bgColor    = pBgColor;
		rectRadius = pRectRadius;
	} // Alerts4P()

	
	/****************************************************************************************
	* 
	*   Disable The Alerts
	*
	****************************************************************************************/
	public void disableAlerts() {
		hideAlerts = true;
	} // disableAlerts()


	/****************************************************************************************
	* 
	*   Display The Alert
	*
	****************************************************************************************/
	public void display() {
		int textWidth;

		// DISABLED
		if (hideAlerts) return;

		// ALERT EXPIRED
		if (millis() > endingMS) return;

		app.pushStyle();
		
		app.rectMode(CORNER);
		app.textAlign(CENTER);
		app.textSize(textSize);
		app.fill(bgColor);

		if (strokeWeight == 0) {
		  app.noStroke();
		} else {
		  app.strokeWeight(strokeWeight);
		  app.stroke(strokeColor);
		} // if(strokeWeight == 0)
			
		textWidth = (int)app.textWidth(currentMsg);
		app.rect(xPos - ((textWidth + horSpacing) >> 1), yPos - (boxHeight >> 1), textWidth + horSpacing, boxHeight, rectRadius);
		app.fill(txtColor);
		app.text(currentMsg, xPos, yPos + (textSize >> 1) - (textSize >> 3));

		app.popStyle();
	} // display()	

	
	/****************************************************************************************
	* 
	*   Enable The Alerts
	*
	****************************************************************************************/
	public void enableAlerts() {
		hideAlerts = false;
	} // enableAlerts()

	
	/****************************************************************************************
	* 
	*   Set An Alert (Default Duration)
	*
	*	@param pMsg			message to display
	*
	****************************************************************************************/
	public void setAlert(String pMsg) {
		currentMsg = pMsg;
		endingMS   = millis() + duration;
	} // setAlert()

	
	/****************************************************************************************
	* 
	*   Set An Alert (Specific Duration)
	*
	*	@param pMsg			message to display
	*	@param pDurationMS	time for displaying the message (millis)
	*
	****************************************************************************************/
	public void setAlert(String pMsg, int pDurationMS) {
		currentMsg = pMsg;
		endingMS   = millis() + pDurationMS;
	} // setAlert()


	/****************************************************************************************
	* 
	*   Set The Background Color
	*
	*	@param pBgColor		background color
	*
	****************************************************************************************/
	public void setBgColor(int pBgColor) {
		bgColor = pBgColor;
	} // setBgColor()	


	/****************************************************************************************
	* 
	*   Change The Default Duration
	*
	*	@param pDuration	new default duration
	*
	****************************************************************************************/
	public void setDuration(int pDuration) {
		duration = pDuration;
	} // setDuration()
	
	
	/****************************************************************************************
	* 
	*   Set The Horizontal Spacing (= Horizontal Padding Inside Of The Popup Box)
	*
	*	@param pHorSpacing	new horizontal spacing
	*
	****************************************************************************************/
	public void setHorSpacing(int pHorSpacing) {
		horSpacing = pHorSpacing;
	} // setHorSpacing()	

	
	/****************************************************************************************
	* 
	*   Set The Default Opacity
	*
	*	@param pOpacity		new default opacity
	*
	****************************************************************************************/
	public void setOpacity(int pOpacity) {
		bgColor = color(255, pOpacity);
	} // setOpacity()  


	/****************************************************************************************
	* 
	*   Set The Radius Of The Popup Rectangle
	*
	*	@param pRectRadius	radius for the corners
	*
	****************************************************************************************/
	public void setRectRadius(int pRectRadius) {
		rectRadius = pRectRadius;
	} // setRectRadius()


	/****************************************************************************************
	* 
	*   Set The Sroke Color For The Popup
	*
	*	@param pStrokeColor	new default stroke color
	*
	****************************************************************************************/
	public void setStrokeColor(int pStrokeColor) {
		strokeColor = pStrokeColor;
	} // setStrokeColor()	

	
	/****************************************************************************************
	* 
	*   Set The Sroke Weight For The Popup (0 = Nostroke)
	*
	*	@param pStrokeWeight	weight of the stroke
	*
	****************************************************************************************/
	public void setStrokeWeight(int pStrokeWeight) {
		strokeWeight = pStrokeWeight;
	} // setStrokeWeight()  

	
	/****************************************************************************************
	* 
	*   Set The Text Color
	*
	*	@param pTxtColor	color of the text
	*
	****************************************************************************************/
	public void setTextColor(int pTxtColor) {
		txtColor = pTxtColor;
	} // setTextColor()	
  
  
	/****************************************************************************************
	* 
	*   Set The Text Size
	*
	*	@param pTextSize	fontsize of the text
	*
	****************************************************************************************/
	public void setTextSize(int pTextSize) {
		textSize   = pTextSize;
		boxHeight  = textSize + 16;
	} // setTextSize()
	
	
	/*********************************************************************************
	 *
	 *	Displays Version Information Of This Library In The Console
	 *
	 *********************************************************************************/
	public void version() {
		System.out.println("Alerts4P v" + version + " [" + releaseDate + "], by Rolf van Gelder (cagewebdev.com)\n");
	} // void version()		
} // Alerts4P